# © Copyright, Fervent | All Rights Reserved
"""
# ==========================
# Exploring MD&A (Text) Data
# ==========================

Are You A Beginner?
We STRONGLY recommend using the .ipynb version instead of this .py version
The .ipynb has more explanatory notes to help and guide you through.

The .py version is largely designed for more intermediate / advanced users of
Python.
"""

# Import package dependencies
import pandas as pd  # for general data wrangling / exploration
import os  # since we're working with files
import matplotlib.pyplot as plt  # for plotting
import seaborn as sns  # for making plots look good
sns.set()  # implementing seaborn chart templates / style

# Get a list of ALL files inside the mda folder
MDA_FILES = os.listdir('../data/mda/')
# Remove the cache file ".DS_Store"
MDA_FILES = [file for file in MDA_FILES if file != '.DS_Store']
# Verifying that the ".DS_Store" is no longer in here.
# NOTE: the file is still in the folder; the code in `In[3]` will NOT delete the actual file.
print(MDA_FILES)

# Check the number of files we have
print(len(MDA_FILES))

# Exploring the file naming convention
print(MDA_FILES[0])

# File Naming Convention:
# CIKidentifier_docTheme_filingDate.txt

# Transform the list of mda filenames into a pandas dataframe
df = pd.DataFrame(MDA_FILES)
# Set the column name to "file_name" instead of the default (0)
df.columns = ['file_name']

# Get the CIK for each firm
df['cik'] = df['file_name'].apply(lambda x: x.split('_')[0])

# Get the filing date for each firm
df['filing_date'] = df['file_name'].apply(
    lambda x: x.split('_')[2].split('.')[0])

# Convert the date strings to pandas timestamps
df['filing_date'] = pd.to_datetime(df['filing_date'], format='%Y-%m-%d')

# Explore the number of filings by year
df.groupby(df['filing_date'].dt.year)[
    'cik'].count().plot(kind='bar', figsize=(12, 8))

# Summary statistics of files
df.groupby('cik')['file_name'].count().describe()

# Explore the number of filings per firm per year
df.groupby(['cik', df['filing_date'].dt.year])['file_name'].count().unstack()

df.groupby(['cik', df['filing_date'].dt.year])[
    'file_name'].count().unstack().describe()

# BONUS: Viewing More Observations
with pd.option_context('display.max_rows', None):
    print(df.groupby(['cik', df['filing_date'].dt.year])
          ['file_name'].count().unstack())
